/*****************************************************************
/
/ File   :   AdmHndlr.c
/ Author :   David Corcoran
/ Date   :   October 15, 1999
/ Purpose:   This handles administrative functions like reset/power.
/            See http://www.linuxnet.com for more information.
/ License:   See file LICENSE
/
******************************************************************/

#include "AdmHndlr.h"
#include "serial.h"

UCHAR Adm_CheckSum( PUCHAR pucBuffer, ULONG ulLength ) {

  int i;
  UCHAR ucXSum = 0;

  for (i=0; i < ulLength; i++) {
    ucXSum ^= pucBuffer[i];
  }

  return ucXSum;

}

ULONG Adm_Initialize( char *pcPort ) {

  if ( IO_InitializePort( 9600, 8, 'N', pcPort ) == 1 ) {
    Adm_SetWWT(25);
    return STATUS_SUCCESS;
  } else {
    return STATUS_UNSUCCESSFUL;
  }
}

ULONG Adm_UnInitialize() {

  if ( IO_Close() == 1 ) {
    return STATUS_SUCCESS;
  } else {
    return STATUS_UNSUCCESSFUL;
  }
}

ULONG Adm_PowerIFD() {

  ULONG rv;
  UCHAR ucPower[3];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ulRxLength = 0;

  ucPower[0] = 0x10;    /* Packet Type */
  ucPower[1] = 0x01;    /* Destination */
  ucPower[2] = 0x12;    /* Speed-9600  */

  rv = Adm_Transmit( ucPower, 3, ucResponse, &ulRxLength );

  return rv;
}

ULONG Adm_UnPowerIFD() {

  ULONG rv;
  UCHAR ucPower[3];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ulRxLength = 0;

  ucPower[0] = 0x11;    /* Packet Type */
  ucPower[1] = 0x01;    /* Destination */

  rv = Adm_Transmit( ucPower, 2, ucResponse, &ulRxLength );

  return rv;
}

ULONG Adm_PowerICC( ULONG ulSlot, PUCHAR pucAtr, PULONG pulAtrLen ) {

  ULONG rv;
  UCHAR ucReset[5];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ucReset[0] = 0x20;    /* Packet Type */
  ucReset[2] = 0x00;    /* Auto        */

  switch( ulSlot ) {
  case 0x01:
    ucReset[1] = 0x03;
    break;
  case 0x02:
    ucReset[1] = 0x02;
    break;
  default:
    return STATUS_UNSUCCESSFUL;
  }

  rv = Adm_Transmit( ucReset, 3, ucResponse, &ulRxLength );

  if ( rv == STATUS_SUCCESS ) {
    memcpy( pucAtr, ucResponse, ulRxLength - 2 );
    *pulAtrLen = ulRxLength - 2;
  } else {
    *pulAtrLen = 0;
  }

  return rv;
}

ULONG Adm_UnPowerICC( ULONG ulSlot ) {

  ULONG rv;
  UCHAR ucReset[5];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ucReset[0] = 0x21;    /* Packet Type */
  ucReset[2] = 0x00;    /* Auto        */

  switch( ulSlot ) {
  case 0x01:
    ucReset[1] = 0x03;
    break;
  case 0x02:
    ucReset[1] = 0x02;
    break;
  default:
    return STATUS_UNSUCCESSFUL;
  }

  rv = Adm_Transmit( ucReset, 3, ucResponse, &ulRxLength );

  return rv;
}


ULONG Adm_ResetICC( ULONG ulSlot, PUCHAR pucAtr, PULONG pulAtrLen ) {

  ULONG rv;
  UCHAR ucReset[5];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ucReset[0] = 0x22;    /* Packet Type */
  ucReset[2] = 0x00;    /* Auto        */

  switch( ulSlot ) {
  case 0x01:
    ucReset[1] = 0x03;
    break;
  case 0x02:
    ucReset[1] = 0x02;
    break;
  default:
    return STATUS_UNSUCCESSFUL;
  }

  rv = Adm_Transmit( ucReset, 3, ucResponse, &ulRxLength );

  if ( rv == STATUS_SUCCESS ) {
    memcpy( pucAtr, ucResponse, ulRxLength - 2 );
    *pulAtrLen = ulRxLength - 2;
  } else {
    *pulAtrLen = 0;
  }

  return rv;
}

ULONG Adm_Transmit( PUCHAR pucTxBuffer, ULONG ulTxLength, 
                    PUCHAR pucRxBuffer, PULONG pulRxLength ) {

  UCHAR ucCommand[MAX_BUFFER_SIZE];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulLength, ulPckRxLen;
  static UCHAR ucSequence = 1;
  UCHAR ucXSum;

  ucCommand[0] = 'A';             /* Add Header Bytes */
  ucCommand[1] = 'S';
  ucCommand[2] = 'H';

  ucCommand[3] = ucSequence;      /* Sequence Number  */
  ucCommand[4] = pucTxBuffer[0];  /* Packet Type      */
  ucCommand[5] = pucTxBuffer[1];  /* Destination      */

  ulLength     = ulTxLength - 2;

  ucCommand[7] = ulLength / 0x10; /* High byte size   */
  ucCommand[6] = ulLength % 0x10; /* Low byte size    */

  if ( ulLength > 0 ) {
    memcpy( &ucCommand[8], &pucTxBuffer[2], ulLength );
  }

  /* Do that Check Sum */
  ucCommand[8+ulLength] = Adm_CheckSum( ucCommand, 8+ulLength );

  ulLength = ulLength + 9;

  if ( IO_Write( ulLength, ucCommand ) == 1 ) {
    if ( IO_Read( 3, ucResponse ) == 1 ) {
      if ( ucResponse[0] == 'A' &&
	   ucResponse[1] == 'S' &&
	   ucResponse[2] == 'R' ) {

	if ( IO_Read( 5, ucResponse ) == 1 ) {
	  ulPckRxLen = ucResponse[3] + (0x10*ucResponse[4]);
	  if ( IO_Read( ulPckRxLen, ucResponse )) {
	    IO_Read( 1, &ucXSum );
	    memcpy( pucRxBuffer, ucResponse, ulPckRxLen );
	    *pulRxLength = ulPckRxLen;
	    if ( ucSequence == 0xFF ) {
	      ucSequence = 1;
	    } else {
	      ucSequence += 1;
	    }
	    
	    return STATUS_SUCCESS;
	  }
	}	
      }
    }
  }

  *pulRxLength = 0;

  return STATUS_DATA_ERROR;
}

ULONG Adm_SetWWT( ULONG ulWWT ) {
  IO_UpdateReturnBlock( ulWWT );
  return STATUS_SUCCESS;
}

ULONG Adm_IsICCPresent( ULONG ulSlot ) {

  ULONG rv;
  UCHAR ucStatus[3];
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  ulRxLength = 0;

  ucStatus[0] = 0x16;    /* Packet Type */
  ucStatus[1] = 0x01;    /* Destination */

  rv = Adm_Transmit( ucStatus, 2, ucResponse, &ulRxLength );

  if ( rv == STATUS_SUCCESS ) {

    switch( ulSlot ) {
    case 0x01:
      if ( ucResponse[0] & 0x10 ) {
	rv = STATUS_SUCCESS;
      } else {
	rv = STATUS_UNSUCCESSFUL;
      }
      break;
    case 0x02:
      if ( ucResponse[0] & 0x40 ) {
	rv = STATUS_SUCCESS;
      } else {
	rv = STATUS_UNSUCCESSFUL;
      }
      break;
    default:
      rv = STATUS_UNSUCCESSFUL;
    }

  }

  return rv;
}

